/*
 * Copyright (C) 2014
 *      Andrew Hayzen <ahayzen@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import Bacon2D 1.0
import Ubuntu.Components 1.1
import QtQuick 2.3
import QtSensors 5.3
import "../components"

Scene {
    height: parent.height
    gravity: Qt.point(0, 20)
    pixelsPerMeter: units.gu(3)
    physics: true
    running: false
    viewport: Viewport {
    }
    width: parent.width

    property int netDepth: units.gu(1)
    property int wallDepth: units.gu(.5)
    property var lastWinner

    signal gameFinished(Character human, Character computer)
    signal roundWon(Character winner)

    onRoundWon: {
        // Increment score and store last winner
        winner.score++;
        gameScene.lastWinner = winner;

        // Check if the game has finished
        gameScene.running = false;

        if (winner.score >= settings.targetScore) {
            gameFinished(human, computer);
        } else {
            pointOverlay.visible = true;
        }
    }

    // Scoring for the players
    LiveScore {
        anchors {
            left: parent.left
        }
        color: human.color
        score: human.score
    }

    LiveScore {
        anchors {
            right: parent.right
        }
        color: computer.color
        score: computer.score
    }

    // Level boundaries
    Wall {
        id: ground
        anchors {
            bottom: parent.bottom
            left: parent.left
            right: parent.right
        }
        height: parent.wallDepth
    }
    Wall {
        id: leftWall
        anchors {
            bottom: parent.bottom
            right: parent.left
        }
        friction: 0
        height: parent.height * 10
        width: parent.wallDepth
    }
    Wall {
        id: rightWall
        anchors {
            bottom: parent.bottom
            left: parent.right
        }
        friction: 0
        height: parent.height * 10
        width: parent.wallDepth
    }

    // Net in centre
    Wall {
        id: centreNet
        anchors {
            bottom: parent.bottom
            horizontalCenter: parent.horizontalCenter
        }
        friction: 0
        height: parent.height / 5
        width: parent.netDepth
    }

    // Top of centre net to prevent ball becoming stuck
    SemiCircleEntity {
        color: "#000"
        friction: 0
        radius: parent.netDepth / 2
        x: centreNet.x
        y: centreNet.y - radius
    }

    // Barrier to prevent player jumping net
    CharacterBarrier {
        anchors {
            bottom: parent.bottom
            horizontalCenter: centreNet.horizontalCenter
        }
        height: parent.height
        width: parent.netDepth
    }

    Rectangle {
        id: volleyBallHint
        anchors {
            top: parent.top
        }
        color: volleyball.color
        height: volleyball.radius / 2
        width: volleyball.radius
        x: volleyball.x - width / 2  // volleyball.x is actually centre
        visible: volleyball.y < y
    }

    // Volley Ball
    Ball {
        id: volleyball
        radius: units.gu(2)
        onContact: {
            if (other === ground) {
                roundWon(x < parent.width / 2 ? computer : human)
            }
        }
    }

    // Character
    Human {
        id: human
        objectName: "human"
    }

    // AI Prediction DEBUG
    Rectangle {
        id: aiPred
        color: "#0F0"
        height: wallDepth
        width: wallDepth
        visible: settings.aiDebugLayer
    }

    Rectangle {
        id: aiPredX
        color: "#F00"
        height: parent.height - wallDepth
        width: wallDepth / 2
        visible: settings.aiDebugLayer
    }

    Rectangle {
        id: aiPredY
        color: "#00F"
        height: wallDepth / 2
        width: parent.width
        visible: settings.aiDebugLayer
    }

    // Enemy
    Computer {
        id: computer
        ball: volleyball
        gravity: gameScene.gravity
        minX: centreNet.x + centreNet.width
        maxX: gameScene.width
        objectName: "computer"
    }

    // Tap controls
    MultiPointTouchArea {
        id: multiTouchArea
        anchors {
            fill: parent
        }
        enabled: !settings.useMotion
        minimumTouchPoints: 1
        maximumTouchPoints: 2
        touchPoints: [
            TouchPoint {
                id: point1
                onPressedChanged: {
                    if (pressed) {
                        if (x < gameScene.width / 3) {
                            human.leftPressed = true
                            human.rightPressed = false
                        } else if (gameScene.width / 3 <= x && x <= gameScene.width - (gameScene.width / 3)) {
                            human.jump()
                        } else {
                            human.leftPressed = false
                            human.rightPressed = true
                        }
                    } else {
                        human.leftPressed = false
                        human.rightPressed = false
                    }
                }
            },
            TouchPoint {
                id: point2
                onPressedChanged: {
                    if (pressed) {
                        human.jump();
                    }
                }
            }
        ]
    }

    // Motion controls
    MouseArea {
        anchors {
            fill: parent
        }
        enabled: settings.useMotion
        onPressed: human.upPressed = true
        onReleased: human.upPressed = false
    }

    OrientationSensor {
        id: orientationSensor
        active: tiltSensor.active
    }

    TiltSensor {
        id: tiltSensor
        active: settings.useMotion && gameScene.running
        dataRate: 16

        function getScalar(amount) {
            // Make a scale from 0 -> 2 (from 0deg -> 40deg)
            var scalar = Math.abs(amount) / 20

            if (scalar > human.scalarLimit) {
                scalar = human.scalarLimit
            }

            return scalar;
        }

        onReadingChanged: {
            if (orientationSensor.reading.orientation === OrientationReading.TopUp) {
                human.leftPressed = reading.yRotation < -settings.motionDeadZone ? getScalar(reading.yRotation) : false
                human.rightPressed = reading.yRotation > settings.motionDeadZone ? getScalar(reading.yRotation) : false
            } else if (orientationSensor.reading.orientation === OrientationReading.TopDown) {
                human.leftPressed = reading.yRotation < -settings.motionDeadZone ? false : getScalar(reading.yRotation)
                human.rightPressed = reading.yRotation > settings.motionDeadZone ? false : getScalar(reading.yRotation)
            } else if (orientationSensor.reading.orientation === OrientationReading.RightUp) {
                human.leftPressed = reading.xRotation < -settings.motionDeadZone ? getScalar(reading.xRotation) : false
                human.rightPressed = reading.xRotation > settings.motionDeadZone ? getScalar(reading.xRotation) : false
            } else if (orientationSensor.reading.orientation === OrientationReading.LeftUp) {
                human.leftPressed = reading.xRotation < -settings.motionDeadZone ? false : getScalar(reading.xRotation)
                human.rightPressed = reading.xRotation > settings.motionDeadZone ? false : getScalar(reading.xRotation)
            }
        }
    }

    // Header
    GameHeader {
        id: gameHeader
        anchors {
            horizontalCenter: parent.horizontalCenter
            top: parent.top
        }
    }

    // Point overlay at the end of a round
    PointOverlay {
        id: pointOverlay
        anchors {
            fill: parent
        }
        visible: false
    }

    PauseOverlay {
        id: pauseOverlay
        anchors {
            fill: parent
        }
        objectName: "pauseOverlay"
        visible: false

        onPause: gameScene.running = false
        onResume: gameScene.running = true
    }

    // Keyboard controls
    // TODO: add scaling
    Keys.onEscapePressed: {
        if (pointOverlay.visible) {
            gameScene.reset();
        } else {
            if (pauseOverlay.visible) {
                pauseOverlay.resume()
            } else  {
                pauseOverlay.pause()
            }
        }
    }

    Keys.onReturnPressed: {
        if (pointOverlay.visible) {
            gameScene.reset();
        }
    }

    Keys.onPressed: {
        if (!event.isAutoRepeat) {
            switch (event.key) {
            case Qt.Key_Left:
                human.leftPressed = 1;
                break;
            case Qt.Key_Right:
                human.rightPressed = 1;
                break;
            case Qt.Key_Up:
                human.upPressed = true;
                break;
            }
        }
    }

    Keys.onReleased: {
        if (!event.isAutoRepeat) {
            switch (event.key) {
            case Qt.Key_Left:
                human.leftPressed = false;
                break;
            case Qt.Key_Right:
                human.rightPressed = false;
                break;
            case Qt.Key_Up:
                human.upPressed = false;
                break;
            }
        }
    }

    // Game has ended restart
    function restart() {
        // Reset scores, timer and lastwinner
        lastWinner = undefined;
        human.score = 0;
        computer.score = 0;
        gameHeader.time = 0;

        reset();  // reset positions
    }

    // Reset the sprites between points
    function reset() {
        pointOverlay.visible = false;
        pauseOverlay.visible = false;

        gameScene.running = true;

        // Zero any velocities
        volleyball.angularVelocity = 0.0;
        volleyball.linearVelocity = Qt.point(0, 0);
        human.linearVelocity = Qt.point(0, 0);
        computer.linearVelocity = Qt.point(0, 0);

        human.leftPressed = false;
        human.rightPressed = false;

        // Reset volleyball
        if (lastWinner === computer) {
            volleyball.x = gameScene.width - gameScene.width / 4;
        } else {
            volleyball.x = gameScene.width / 4;
        }

        volleyball.y = gameScene.height / 2;

        // Reset human
        human.x = (gameScene.width / 4) - human.radius;
        human.y = gameScene.height - human.radius;

        // Reset computer
        computer.x = gameScene.width - (gameScene.width / 4) - human.radius;
        computer.y = gameScene.height - human.radius;
    }
}
